/****************************************************************************
 This file is a part of Purp (Pugo RPM) - a ncurses-based RPM-handler
 Copyright (C) 1998-1999 -- Anders Karlsson <pugo@lysator.liu.se>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         
 
 For latest information and version of Purp, look at the Purp homepage:
 <http://www.lysator.liu.se/purp/>
 
 Please send comments and bug-reports to: <pugo@lysator.liu.se>
****************************************************************************/

#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include "main.h"
#include "linklist.h"
#include "dir.h"
#include "tools.h"

#include "panel.h"
#include "textwindow.h"
#include "cmdwindow.h"

extern listelement *listpointer[];
extern int new_option[];
extern char *availdir;
extern short showingMainInterface;   /* 1 if the main interface is displayed */


/****************************************************************************
 * ListPackageFile
 * 
 * Lists all files in the specified package in a scrollable window
 ****************************************************************************/

void ListPackageFiles( listelement *lListpointer, int panel, int option )
{
   listelement *tmpPtr;
   int offset, i;
   Header h; 
   int_32 count, type;
   char tmpStr[ 256 ];
   char *name, * version, * release;
   char ** fileList;
   int_32 * fileSizeList;
   int isSource;
   short ok = 0;
   static FD_t fd;    
   rpmdb db = NULL;
   
   /* Open the RPM-database */
   purp_rpmdbOpen( &db );


   showingMainInterface = 0;
   tmpPtr = ReturnItemNo( lListpointer, option );

   if( panel == INSTALLED )
     {
	offset = FindPackageOffset( tmpPtr );

	if( offset >= 0 )
	  {
	     h = rpmdbGetRecord( db, offset );
	     ok = 1;
	  }
     }
   else
     {
	if ((fd = fdOpen( tmpPtr->filename, O_RDONLY, 0)) >= 0) 
	  {
	     rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL); 
	     fdClose(fd);   
	     ok = 1;
	  }
     }
     

   if( ok )
     {
	headerGetEntry( h, RPMTAG_NAME, &type, (void **) &name, &count ); 
	headerGetEntry( h, RPMTAG_VERSION, &type, (void **) &version, &count ); 
	headerGetEntry( h, RPMTAG_RELEASE, &type, (void **) &release, &count ); 
	
	sprintf( tmpStr, "Files in package %s-%s-%s", name, version, release );
	
	textwin *filesWin;
	filesWin = new textwin( tmpStr, "Use arrow-keys to navigate, any other to continue.", "", 0 , NOWRAP);
	
	if( headerGetEntry( h, RPMTAG_FILENAMES, &type, (void **) &fileList, &count ))
	  {
	     headerGetEntry(h, RPMTAG_FILESIZES, &type,
			    (void **) &fileSizeList, &count); 
	     
	     filesWin->MallocTextBuffer( count );
	     
	     for( i = 0; i < count; i++ )
	       {
		  sprintf( tmpStr, "%10d  %s\n", fileSizeList[ i ], fileList[ i ] );
		    
		  filesWin->SetTextBufferLine( i, tmpStr );
		  filesWin->AddStr( tmpStr, NOWRAP );
	       }
	  }


	filesWin->HandleEvents();
	delete filesWin;
	
	headerFree( h ); 
     }
   
   if(db) rpmdbClose( db );
   CheckResize();
}

/****************************************************************************
 * ListPackageScripts
 * 
 * Lists all scripts in the specified package in a scrollable window
 ****************************************************************************/

void ListPackageScripts( listelement *lListpointer, int panel, int option )
{
   listelement *tmpPtr;
   int offset, i;
   Header h; 
   int_32 count, type;
   char tmpStr[2048] = "";
   int isSource;
   FD_t fd;    
   short ok = 0;
   
   char *name[128], *version[128], *release[128];
   char *prein;
   char *postin;
   char *preun;
   char *postun;
   char *verify;

   rpmdb db = NULL;
   
   /* Open the RPM-database */
   purp_rpmdbOpen( &db );


   showingMainInterface = 0;

   tmpPtr = ReturnItemNo( lListpointer, option );


   if( panel == INSTALLED )
     {
	offset = FindPackageOffset( tmpPtr );

	if( offset >= 0 )
	  {
	     
	     h = rpmdbGetRecord( db, offset );	     

	     ok = 1;
	  }
     }
   else
     {
	if ((fd = fdOpen( tmpPtr->filename, O_RDONLY, 0)) >= 0) 
	  {
	     rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL); 
	     fdClose(fd);   
	     ok = 1;
	  }
     }

   if( ok )
     {
	headerGetEntry( h, RPMTAG_NAME, &type, (void **) &name, &count ); 
	headerGetEntry( h, RPMTAG_VERSION, &type, (void **) &version, &count ); 
	headerGetEntry( h, RPMTAG_RELEASE, &type, (void **) &release, &count ); 

	sprintf( tmpStr, "Scripts in package %s-%s-%s", *name, *version, *release );

	textwin *filesWin;
	filesWin = new textwin( tmpStr, "Use arrow-keys to navigate, any other to continue.", "", 0, NOWRAP );

	sprintf( tmpStr, "\nPre-install script:\n-------------------------\n" );
	filesWin->AddStr( tmpStr, NOWRAP );
	headerGetEntry( h, RPMTAG_PREIN, &type, (void **) &prein, &count );
	sprintf( tmpStr, "%s\n", (prein == NULL) ? "(None)" : prein );
	filesWin->AddStr( tmpStr, WRAP );
	
	sprintf( tmpStr, "\nPost-install script:\n--------------------------\n" );
	filesWin->AddStr( tmpStr, NOWRAP );
	headerGetEntry( h, RPMTAG_POSTIN, &type, (void **) &postin, &count );		
	sprintf( tmpStr, "%s\n", (postin == NULL) ? "(None)" : postin );
	filesWin->AddStr( tmpStr, WRAP );
	
	sprintf( tmpStr, "\nPre-uninstall script:\n---------------------------\n" );
	filesWin->AddStr( tmpStr, NOWRAP );
	headerGetEntry( h, RPMTAG_PREUN, &type, (void **) &preun, &count );
	sprintf( tmpStr, "%s\n", (preun == NULL) ? "(None)" : preun );
	filesWin->AddStr( tmpStr, WRAP );
	
	sprintf( tmpStr, "\nPost-uninstall script:\n----------------------------\n" );
	filesWin->AddStr( tmpStr, NOWRAP );
	headerGetEntry( h, RPMTAG_POSTUN, &type, (void **) &postun, &count );
	sprintf( tmpStr, "%s\n", (postun == NULL) ? "(None)" : postun );
	filesWin->AddStr( tmpStr, WRAP );
	
	sprintf( tmpStr, "\nVerify script:\n----------------------------\n" );
	filesWin->AddStr( tmpStr, NOWRAP );
	headerGetEntry( h, RPMTAG_VERIFYSCRIPT, &type, (void **) &verify, &count );
	sprintf( tmpStr, "%s\n", (verify == NULL) ? "(None)" : verify );
	filesWin->AddStr( tmpStr, WRAP );

	/* Wait until user presses a key.. */
	filesWin->HandleEvents();
	
	/* Free the RPM-header */
	headerFree( h );
	delete filesWin;
     }
   
   if(db) rpmdbClose( db );
   CheckResize();
}


/****************************************************************************
 * ListPackageDependencies
 * 
 * Lists dependencies for the specified package in a scrollable window
 ****************************************************************************/

void ListPackageDependencies( listelement *lListpointer, int panel, int option )
{
   listelement *tmpPtr;
   int offset, i;
   Header h; 
   int_32 count, type;
   char tmpStr[ 256 ];
   char *name, * version, * release;
   char **provides, **requires;
   int isSource;
   short ok[2] = {0,0};
   int lines[2], line;
   FD_t fd;    
   rpmdb db = NULL;
   
   /* Open the RPM-database */
   purp_rpmdbOpen( &db );

   showingMainInterface = 0;

   tmpPtr = ReturnItemNo( lListpointer, option );

   if( panel == INSTALLED )
     {
	offset = FindPackageOffset( tmpPtr );

	if( offset >= 0 )
	  {
	     h = rpmdbGetRecord( db, offset );
	     ok[0] = 1;
	  }
     }
   else
     {
	if ((fd = fdOpen( tmpPtr->filename, O_RDONLY, 0)) >= 0) 
	  {
	     rpmReadPackageHeader(fd, &h, &isSource, NULL, NULL); 
	     fdClose(fd);   
	     ok[0] = 1;
	  }
     }

   if( ok[0] )
     {
	headerGetEntry( h, RPMTAG_NAME, &type, (void **) &name, &count ); 
	headerGetEntry( h, RPMTAG_VERSION, &type, (void **) &version, &count ); 
	headerGetEntry( h, RPMTAG_RELEASE, &type, (void **) &release, &count ); 

	ok[0] = headerGetEntry( h, RPMTAG_PROVIDES, &type, (void **) &provides, &count ); 
	  lines[0] = count;
	ok[1] = headerGetEntry( h, RPMTAG_REQUIRENAME, &type, (void **) &requires, &count ); 
	  lines[1] = count;

	sprintf( tmpStr, "Dependenices for package %s-%s-%s", name, version, release );
	
	textwin *depWin;
	depWin = new textwin( tmpStr, "Use arrow-keys to navigate, any other to continue.", "", 0 , NOWRAP);
	
	if( lines[0] == 0 ) lines[0]++;
	if( lines[1] == 0 ) lines[1]++;
	depWin->MallocTextBuffer( lines[0] + lines[1] + 6 );

	line = 0;
	       	
	depWin->SetTextBufferLine( line, "\n" ); 
	depWin->AddStr( "\n", NOWRAP ); 
	line++;  
	
	depWin->SetTextBufferLine( line, "Provides:\n" );
	depWin->AddStr( "Provides:\n", NOWRAP );
	line++;  
	
	depWin->SetTextBufferLine( line, "----------------------------\n" );
	depWin->AddStr( "----------------------------\n", NOWRAP );
	line++;  
	
	if( ok[0] && (lines[0] > 0) )
	  {
	     for( i = 0; i < lines[0]; i++, line++ )
	       {
		  sprintf( tmpStr, "%s\n", (provides[i] == NULL) ? "(None)\n" : provides[i] );
		  depWin->SetTextBufferLine( line, tmpStr );
		  depWin->AddStr( tmpStr, NOWRAP );
	       }
	  }
	else
	  {
	     depWin->AddStr( "(None)\n", NOWRAP );
	     depWin->SetTextBufferLine( line, "(none)\n" );
	     line++;
	  }
	
	depWin->AddStr( "\n", NOWRAP );
	depWin->SetTextBufferLine( line, "\n" );
	line++;

	depWin->SetTextBufferLine( line, "Requires:\n" );
	depWin->AddStr( "Requires:\n", NOWRAP );
	line++;  
	
	depWin->SetTextBufferLine( line, "----------------------------\n" );
	depWin->AddStr( "----------------------------\n", NOWRAP );
	line++;  

	if( ok[1] && (lines[1] > 1) )
	  {
	     for( i = 0; i < lines[1]; i++, line++ )
	       {
		  sprintf( tmpStr, "%s\n", (requires[i] == NULL) ? "(None)\n" : requires[i] );
		  depWin->SetTextBufferLine( line, tmpStr );
		  depWin->AddStr( tmpStr, NOWRAP );
	       }
	  }
	else
	  {
	     depWin->AddStr( "(none)\n", NOWRAP );
	     depWin->SetTextBufferLine( line, "(none)\n" );
	     line++;
	  }

	depWin->HandleEvents();

	headerFree( h ); 
	delete depWin;
     }

   if(db) rpmdbClose( db );
   CheckResize();
}

