/****************************************************************************
 This file is a part of Purp (Pugo RPM) - a ncurses-based RPM-handler
 Copyright (C) 1998-1999 -- Anders Karlsson <pugo@lysator.liu.se>

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         
 
 For latest information and version of Purp, look at the Purp homepage:
 <http://www.lysator.liu.se/purp/>
 
 Please send comments and bug-reports to: <pugo@lysator.liu.se>
****************************************************************************/
   
#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "textwindow.h"
#include "linklist.h"
#include "main.h"
#include "tools.h"

extern WINDOW    *win1, *win2, *infowin;
extern int        width, height, gMaxoptions[];
extern int        highest_item_on_list[];
extern char       dirstring[2][100];
extern int        panel;

extern listelement *listpointer[2];
extern int       highest_item_on_list[];
extern int       new_option[];


textwin::textwin( char *ltitle, 
		  char *lfooter, 
	          char *text,
 		  int  bottommargin,
		  bool wrap )
{
   char tmpStr[66];
   int lX;
   
   x = 0; 
   y = 1;
   
   top = 0;
   height = LINES - bottommargin - top;
   left = 0;
   width = COLS - left;
   
   strcpy( title, ltitle );
   strcpy( footer, lfooter );

   win = newwin( height, width, top, left );

   if( win == NULL )
    {   
       fprintf( stderr, "Error: Unable to create text-window!\n" );
       endwin();
       exit(0);
    }

   wbkgd( win, COLOR_PAIR(1) );
   wattrset( win, COLOR_PAIR(1) );
   werase( win );   

   /* Header */
   wmove( win, top, left );
   whline( win, 0, width );

   if( strlen( title ) > 0 )
     {
	sprintf( tmpStr, " %s ", title );
	mvwaddch( win, top, left + 2, ACS_RTEE );
	mvwaddstr( win, top, left + 3, tmpStr );
	mvwaddch( win, top, strlen( tmpStr ) + left + 3, ACS_LTEE );
     }
   
   /* Footer */
   wmove( win, top + height - 1, left );
   whline( win, 0, width );

   if( strlen( footer ) > 0 ) 
     {
	sprintf( tmpStr, " %s ", footer );
	lX = width - strlen( footer ) - 6;
	mvwaddch( win, top + height - 1, lX, ACS_RTEE );
	mvwaddstr( win, top + height - 1, lX + 1, tmpStr );
	mvwaddch( win, top + height - 1, left + width - 3, ACS_LTEE );
     }

   textbuffer = NULL;
   
   /* Add initial text */
   if( strlen( text ) > 0 )
     AddStr( text, wrap );
   
   /* At the beginning the first line are at top of the window */
   topline = 0;
   bufferlength = 0;
   insertPos = 0;
   
   wrefresh( win );
}


/* Destructor for the window */
textwin::~textwin( void )
{
   int count;
   #ifdef DEBUG_TO_TERM
     TermWrite( "Destruerar textwin\n" );
   #endif

     if( bufferlength > 0 )
       {
	  /* First free all lines */
	  for( count=0; count < bufferlength; count++ )
	    {
	       if( textbuffer[ count ] != NULL )
		 free( (char *)textbuffer[ count ] );
	    }
	  
	  /* And the free the linepointers.. */
	  if( textbuffer != NULL )
	    free( textbuffer );
       }
}



/* Add text to the textwindow */
void textwin::AddStr( char * text, bool wrap )
{
   char ch;
   char *tmpPtr = text;
   
   wmove( win, top + y, left + x );
   
   while( ((ch = *tmpPtr++) != 0) && ( y <= height - 2) )
     {
	if( ch == '\n' )
	  {
	     y++;
	     x = left;
	     wmove( win, top + y, left );
	  }
	else
	  {
	     waddch( win, ch );
	     x++;
	     if( x >= width )
	       {
		  if( wrap )
		    {
		       y++;
		       x = left;
		       if( y <= height - 2 )
			 wmove( win, y, x );
		    }
		  else
		    {
		       y++;
		       x = left;
		       tmpPtr = strchr( tmpPtr, '\n' ) + 1;	
		       if( y <= height - 2 )
			 wmove( win, y, x );
		    }
	       }
	  }
     }
   wrefresh( win );
}


/* Write text at specified line in the window.. */
void textwin::WriteLine( int line, char * text )
{
   char ch;
   char *tmpPtr = text;
   
   x = left;
   y = line;
   wmove( win, top + y, left + x );
      
   while( ((ch = *tmpPtr++) != 0) && (ch != '\n') && (x < width) )
     {
	waddch( win, ch );
	x++;
     }

   wrefresh( win );
}


/* Eat key-commands from the user.. */
void textwin::HandleEvents( )
{
   int key, count, lines;
   bool eventloop=1;

   wsetscrreg( win, top+1, height-2 );
   scrollok(win, TRUE);
   
   while( eventloop )
     {
	key = getch();

	switch(key)
	  {
	     
	   /* Up we go.. */  
	   case KEY_UP:
	   case '8':
             #ifdef DEBUG_TO_TERM
	         TermWrite( "User pressed 'up'\n" );
             #endif
	       if( topline > 0 )
	       {
		  wscrl( win, -1 );
		  WriteLine( top + 1, textbuffer[ topline - 1 ] );
		  wrefresh( win );
		  topline--;
	       }
	   break;

	     
	   /* Down.. Down.. Down.. */
	   case KEY_DOWN:
	   case '2':
             #ifdef DEBUG_TO_TERM
	       TermWrite( "User pressed 'down'\n" );
             #endif
	     if( (topline + height - 2) < bufferlength )
	       {
		  wscrl( win, 1 );
		  WriteLine( height-2, textbuffer[ topline + height - 2 ] );
		  wrefresh( win );
		  topline++;
	       }
	     break;
	     
	     
	     /* Page up.. */
	   case KEY_PPAGE:
             #ifdef DEBUG_TO_TERM
	       TermWrite( "User pressed 'page-up'\n" );
             #endif
	     if( topline > 0 )
	       {
		  if( (topline - height - 2) < 0 )
		    lines = topline;
		  else
		    lines = height-3;

		  wscrl( win, -lines );
		  topline -= lines;

		  for( count = 0; count < lines; count++ )
		    WriteLine( count+1, textbuffer[ topline + count ] );

		  wrefresh( win );
	       }
	     break;
	     
	     
	     /* Page down */
	   case KEY_NPAGE:
           #ifdef DEBUG_TO_TERM
	       TermWrite( "User pressed 'page-down'\n" );
           #endif
	       
	     if( (topline + height - 2) < bufferlength )
	       {
		  wscrl( win, height-3 );
		  topline += height-3;

		  if( (topline+height-3) > bufferlength )
		    lines = bufferlength-topline-1;
		  else
		    lines = height-3;


		  for( count = 0; count < lines; count++ )
		    WriteLine( top+count+2, textbuffer[ topline + count + 1 ] );

		  wrefresh( win );
	       }
	     break;


           #ifdef DEBUG_TO_TERM
	     /* debug - key */
 	   case 'd':
	       TermWrite( "User pressed 'd' (dump debug-info)\n" );
	       for( count=0; count <= bufferlength; count++ )
	       {
		  TermWrite( "%s\n", textbuffer[ count ] );
	       }
           break;
            #endif
	     
	     
	   default:
           #ifdef DEBUG_TO_TERM
	       TermWrite( "User pressed another key.. Remove textwindow\n" );
           #endif
	     eventloop = 0;
	  }
     }
}


/**************************************************************
 * textwin::MallocTextBuffer
 * 
 * Allocates memory for the text-lines.
 * The lines are only pointers to strings.
 * All pointers are initially set to NULL.
 **************************************************************/
int textwin::MallocTextBuffer( long lines )
{
   int count;
   
   textbuffer = (char **)xmalloc( lines * sizeof( char * ) );

   /* Set new pointers to NULL */
   for( count=0; count < lines; count++ )
     textbuffer[count] = NULL;

   bufferlength = lines;
}


/**************************************************************
 * textwin::ReallocTextBuffer
 * 
 * Allocates more/less memory for the textbuffer.
 * The old pointers will be copied, the new set to NULL.
 **************************************************************/
int textwin::ReallocTextBuffer( long lines )
{
   int count;
   char ** newbuffer;
   
   newbuffer = (char **)xmalloc( lines * sizeof( char * ) );
   
   for( count=0; count < lines; count++ )
     {
	if( count < bufferlength )
	  {
	     /* Copy the old string-pointer */
	     newbuffer[count] = textbuffer[count];
	  }
	else
	  {
	     /* Set new pointers to NULL */
	     newbuffer[count] = NULL;
	  }
     }
   bufferlength = lines;
}


/**************************************************************
 * textwin::AddTextBuffer
 * 
 * Adds text to the buffer, positioned at 'insertPos'
 * 
 * If 'wrap' is true then '\n' will break the line
 * and the rest will appear on the next buffer-line.
 **************************************************************/
void textwin::AddTextBuffer( char * text, short wrap )
{
   char * textline;
   char tmpstr[256] = ""; 

   strcpy( tmpstr, text );

   textline = strtok( tmpstr, "\n" );
   
   while( textline != NULL )
     {
	SetTextBufferLine( insertPos, textline );
	textline = strtok( NULL, "\n" );
     }
}


/**************************************************************
 * textwin::SetTextBufferLine
 * 
 * Set line 'line' in the textbuffer to 'textline'
 **************************************************************/
void textwin::SetTextBufferLine( int line, char * textline )
{
   if( line <= bufferlength )
     {
	textbuffer[ line ] = (char *)xmalloc( strlen( textline ) + 16 );
	strcpy( textbuffer[ line ], textline );
	
	/* Set insert-position to next line */
	insertPos = line + 1;
     }
}
